/* @(#)serrmsg.c	1.2 01/11/09 Copyright 1985,2000 J. Schilling */
/*
 *	Routines for printing command errors
 *
 *	Copyright (c) 1985,2000 J. Schilling
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <mconfig.h>
#include <unixstd.h>		/* include <sys/types.h> try to get size_t */
#include <stdio.h>		/* Try again for size_t	*/
#include <stdxlib.h>		/* Try again for size_t	*/
#include <standard.h>
#include <stdxlib.h>
#include <vadefs.h>
#include <strdefs.h>
#include <schily.h>
#ifndef	HAVE_STRERROR
extern	char	*sys_errlist[];
extern	int	sys_nerr;
#endif

EXPORT	int	serrmsg		__PR((char *buf, size_t maxcnt, const char *, ...));
#ifdef __VMS
#include <errno.h>
EXPORT	int	serrmsgno	__PR((int, int, char *buf, size_t maxcnt, const char *, ...));
LOCAL	int	_serrmsg	__PR((int, int, char *buf, size_t maxcnt, const char *, va_list));
#else
EXPORT	int	serrmsgno	__PR((int, char *buf, size_t maxcnt, const char *, ...));
LOCAL	int	_serrmsg	__PR((int, char *buf, size_t maxcnt, const char *, va_list));
#endif

/* VARARGS1 */
EXPORT int
#ifdef	PROTOTYPES
serrmsg(char *buf, size_t maxcnt, const char *msg, ...)
#else
serrmsg(buf, maxcnt, msg, va_alist)
	char	*buf;
	size_t	maxcnt;
	char	*msg;
	va_dcl
#endif
{
	va_list	args;
	int	ret;

#ifdef	PROTOTYPES
	va_start(args, msg);
#else
	va_start(args);
#endif
#ifdef __VMS
	ret = _serrmsg(geterrno(), getvmserrno(), buf, maxcnt, msg, args);
#else
	ret = _serrmsg(geterrno(), buf, maxcnt, msg, args);
#endif
	va_end(args);
	return (ret);
}

/* VARARGS2 */
#ifdef	PROTOTYPES
EXPORT int
#ifdef __VMS
serrmsgno(int err, int vmserr, char *buf, size_t maxcnt, const char *msg, ...)
#else
serrmsgno(int err, char *buf, size_t maxcnt, const char *msg, ...)
#endif
#else
#ifdef __VMS
serrmsgno(err, vmserr, buf, maxcnt, msg, va_alist)
	int	err;
	int	vmserr;
	char	*buf;
	size_t	maxcnt;
	char	*msg;
	va_dcl
#else
serrmsgno(err, buf, maxcnt, msg, va_alist)
	int	err;
	char	*buf;
	size_t	maxcnt;
	char	*msg;
	va_dcl
#endif
#endif
{
	va_list	args;
	int	ret;

#ifdef	PROTOTYPES
	va_start(args, msg);
#else
	va_start(args);
#endif
#ifdef __VMS
	ret = _serrmsg(err, vmserr, buf, maxcnt, msg, args);
#else
	ret = _serrmsg(err, buf, maxcnt, msg, args);
#endif
	va_end(args);
	return (ret);
}

#ifdef __VMS
LOCAL int
_serrmsg(err, vmserr, buf, maxcnt, msg, args)
	int		err;
	int		vmserr;
	char		*buf;
	size_t		maxcnt;
	const char	*msg;
	va_list		args;
#else
LOCAL int
_serrmsg(err, buf, maxcnt, msg, args)
	int		err;
	char		*buf;
	size_t		maxcnt;
	const char	*msg;
	va_list		args;
#endif
{
	int	ret;
	char	errbuf[20];
	char	*errnam;
	char	*prognam = get_progname();
	
	if (err < 0) {
		ret = js_snprintf(buf, maxcnt, "%s: %r", prognam, msg, args);
	} else {
		errnam = errmsgstr(err);
		if (errnam == NULL) {

#ifdef __VMS
                        if ( err == EVMSERR )
			  (void)js_snprintf(errbuf, sizeof (errbuf),
						"Error %d (VMS=0x0%08.8x)", err, vmserr );
                        else
			  (void)js_snprintf(errbuf, sizeof (errbuf),
						"Error %d", err );
#else
			(void)js_snprintf(errbuf, sizeof (errbuf),
						"Error %d", err);
#endif
			errnam = errbuf;
		}
		ret = js_snprintf(buf, maxcnt,
				"%s: %s. %r", prognam, errnam, msg, args);
	}
	return (ret);
}
